//---------------------------------------------------------------------------
// see mixmastdlg.cpp for notes on load-time vs run-time binding
// MixDir() is in global.cpp
#include <vcl\vcl.h>
#include <vcl\registry.hpp>
#pragma hdrstop

#include <assert.h>
#include "Mixmaster.h"
#include "console.h"  // for RunConsoleApp
//#include "global.h" // for AppDir
#include "wiper.h"
#include "fileman.h"

HINSTANCE hMixLib=NULL;
//---------------------------------------------------------------------------
bool _fastcall HasMixmaster()
{
   return hMixLib>0;
}
//---------------------------------------------------------------------------
bool __fastcall HasMix()
{
   return hMixLib>0;
}
//---------------------------------------------------------------------------
AnsiString __fastcall MixDir()
{
   AnsiString mixdir = "";
   TRegistry *reg = new TRegistry();
   if (reg->OpenKey(MixKey,false)) mixdir = reg->ReadString("MixDir");
   delete reg;
   if ( mixdir.IsEmpty()) mixdir = getenv("MIXDIR");
   if (!mixdir.IsEmpty() && mixdir[mixdir.Length()]!='\\') mixdir += "\\";
   return mixdir.LowerCase();
}
//---------------------------------------------------------------------------
int buf_readfile(BUFFER *buffer, char *filename)
{
   int err = -2;
   buf_reset(buffer);
   TFileStream *f = new TFileStream(filename,fmOpenRead);
   if (f) {
      char buf[BUFSIZE] = { 0 };
      int n;
      err = -1;
      for (;;) {
         n = f->Read(buf,BUFSIZE);
         if (n > 0) err = 0;
         buf_append(buffer, (byte*)buf, n);
         if (n < BUFSIZE) break;

         if (buffer->length > BUFFER_MAX) {
            MsgBox("Message file too large. Giving up.","Error",MB_OK);
            //errlog(ERRORMSG, "Message file too large. Giving up.\n");
            return(1);
         }
      }
      delete f;
   }
   return err;
}
//---------------------------------------------------------------------------
int buf_writefile(BUFFER *buffer, char *filename, char* mode) // mode = "wb" or "wt"
{
   int err = -2;
   FILE *f = fopen(filename,mode);
   if (f) { err = -1;
      int n = (int)fwrite(buffer->data, 1, buffer->length, f)==buffer->length ? 0:-1;
      if (n>0) err=0;
      fclose(f);
   }
   return err;
/* int cnt=0;
   TFileStream *f = new TFileStream(MixTmp,fmCreate);
   if (f)
   {
      cnt = f->Write(buffer->data,buffer->length);
      delete f;
   }
   return cnt;
*/
}
//---------------------------------------------------------------------------
BUFFER* __fastcall buf_new(void)
{
   if (!hMixLib) return NULL;
   BUFFER* ret = NULL; BUF_NEW bufnew;
   if ((bufnew = (BUF_NEW)GetProcAddress(hMixLib,"buf_new"))!=NULL)
      ret = (bufnew)();
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_free(BUFFER *buf)
{
   if (!hMixLib) return NULL;
   int ret = NULL; BUF_FREE buffree;
   if ((buffree = (BUF_FREE)GetProcAddress(hMixLib,"buf_free"))!=NULL)
      ret = (buffree)(buf);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_reset(BUFFER *buf)
{
   if (!hMixLib) return NULL;
   int ret = NULL; BUF_RESET bufreset;
   if ((bufreset = (BUF_RESET)GetProcAddress(hMixLib,"buf_reset"))!=NULL)
      ret = (bufreset)(buf);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_append(BUFFER *buffer, byte *mess, int len)
{
   if (!hMixLib) return NULL;
   int ret = NULL; BUF_APPEND bufappend;
   if ((bufappend = (BUF_APPEND)GetProcAddress(hMixLib,"buf_append"))!=NULL)
      ret = (bufappend)(buffer,mess,len);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_sets(BUFFER *buffer, char *s)
{
   if (!hMixLib) return NULL;
   int ret = NULL; BUF_SETS bufsets;
   if ((bufsets = (BUF_SETS)GetProcAddress(hMixLib,"buf_sets"))!=NULL)
      ret = (bufsets)(buffer,s);
   return ret;
}
//---------------------------------------------------------------------------
void __fastcall buf_move(BUFFER *dest, BUFFER *src)
{
   if (!hMixLib) return;
   BUF_MOVE bufmove;
   if ((bufmove = (BUF_MOVE)GetProcAddress(hMixLib,"buf_move"))!=NULL)
       (bufmove)(dest,src);
}
//---------------------------------------------------------------------------
int __fastcall buf_write(BUFFER *buffer, FILE *out)
{
   if (!hMixLib) return NULL;
   int ret = NULL; BUF_WRITE bufwrite;
   if ((bufwrite = (BUF_WRITE)GetProcAddress(hMixLib,"buf_write"))!=NULL)
      ret = (bufwrite)(buffer,out);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
		BUFFER *feedback)
{
   if (!hMixLib) return NULL;
   int ret=NULL; MIX_ENCRYPT mixencrypt;
   if ((mixencrypt = (MIX_ENCRYPT)GetProcAddress(hMixLib,"mix_encrypt"))!=NULL)
      ret = (mixencrypt)(type,message,chain,numcopies,feedback);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall mix_encrypt_exe(int type, BUFFER *message, char *chain, int numcopies,
   BUFFER *feedback, bool *cancel, TProgressBar *progressbar)
{
   int ret=NULL; if (!hMixLib) return ret;

   AnsiString TheChain,TheCopies;
   TStringList *output = new TStringList();

   TheChain = chain!=NULL && strlen(chain)>0 ? "-l \""+AnsiString(chain)+"\" ":AnsiString("");
   TheCopies = numcopies > 1 ? "-c "+AnsiString(numcopies)+" " : AnsiString("");

   buf_writefile(message,MixTmp.c_str(),"wt");
   ret = RunConsoleApp(MixDir()+MixExe+" "+TheCopies+TheChain+MixTmp,output,SW_HIDE,
      cancel, progressbar);
   buf_sets(feedback,output->Text.c_str());
   simplewipe(MixTmp,10);
   delete output;
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall mix_send()
{
   if (!hMixLib) return NULL;
   int ret = NULL; MIX_SEND mixsend;
   if ((mixsend = (MIX_SEND)GetProcAddress(hMixLib,"mix_send"))!=NULL)
      ret = (mixsend)();
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_mailenc(int mode, BUFFER *message, char *sigid,
		BUFFER *pass, char *pubring, char *secring)
{
   if (!hMixLib) return NULL;
   int ret = NULL; PGP_MAILENC_BUF pgpmailenc;
   if ((pgpmailenc = (PGP_MAILENC_BUF)GetProcAddress(hMixLib,"pgp_mailenc"))!=NULL)
      ret = (pgpmailenc)(mode,message,sigid,pass,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
		BUFFER *pass, char *pubring, char *secring)
{
   if (!hMixLib) return NULL;
   int ret = NULL; PGP_ENCRYPT_BUF pgpencrypt;
   if ((pgpencrypt = (PGP_ENCRYPT_BUF)GetProcAddress(hMixLib,"pgp_encrypt"))!=NULL)
      ret = (pgpencrypt)(mode,message,encr,sigid,pass,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig, char *pubring,
		char *secring)
{
   if (!hMixLib) return NULL;
   int ret = NULL; PGP_DECRYPT pgpdecrypt;
   if ((pgpdecrypt = (PGP_DECRYPT)GetProcAddress(hMixLib,"pgp_decrypt"))!=NULL)
      ret = (pgpdecrypt)(message,pass,sig,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall rnd_add(byte *b, long len)
{
   if (!hMixLib) return NULL;
   int ret = NULL; RND_ADD rndadd;
   if ((rndadd = (RND_ADD)GetProcAddress(hMixLib,"rnd_add"))!=NULL)
      ret = (rndadd)(b,len);
   return ret;
}
//---------------------------------------------------------------------------
void __fastcall rnd_time()
{
   if (!hMixLib) return;
   RND_TIME rndtime;
   if ((rndtime = (RND_TIME)GetProcAddress(hMixLib,"rnd_time"))!=NULL)
      (rndtime)();
}
//---------------------------------------------------------------------------
int __fastcall rnd_mouse(UINT i, WPARAM w, LPARAM l)
{
   if (!hMixLib) return -1;
   int ret = NULL; RND_MOUSE rndmouse;
   if ((rndmouse = (RND_MOUSE)GetProcAddress(hMixLib,"rnd_mouse"))!=NULL)
      ret = (rndmouse)(i,w,l);
   return ret;
}
//---------------------------------------------------------------------------


